import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {GamesystemCondition} from "../../../../project/game-model/interactions/condition/GamesystemCondition";
import {MatSnackBar} from "@angular/material/snack-bar";
import {MatTableDataSource} from "@angular/material/table";
import {CharacterDependency} from "../../../../project/game-model/interactions/CharacterDependency";
import {Gamesystem} from "../../../../project/game-model/gamesystems/Gamesystem";

@Component({
  selector: 'app-gamesystem-condition-editor',
  templateUrl: './gamesystem-condition-editor.component.html',
  styleUrl: './gamesystem-condition-editor.component.scss'
})
export class GamesystemConditionEditorComponent implements OnInit{

  @Input() gamesystemConditions: GamesystemCondition[] = []
  @Input() gamesystems: Gamesystem<any, any>[] = []
  @Output("onConditionUpdate") gamesytemConditionChangeEmitter: EventEmitter<GamesystemCondition> = new EventEmitter();
  @Output("onConditionDelete") gamesystemConditionDeleteEmitter: EventEmitter<GamesystemCondition> = new EventEmitter();
  editedCondition: GamesystemCondition | undefined
  addedCondition: GamesystemCondition | undefined

  datasource: MatTableDataSource<GamesystemCondition> = new MatTableDataSource<GamesystemCondition>();
  displayedColumns: string[] = ["characterDependency", "targetGamesystem", "requieredState", "edit", "delete"]

  constructor(private snackbar: MatSnackBar) {
  }

  ngOnInit() {
    this.datasource.data = this.gamesystemConditions;
  }

  editCondition(condition: GamesystemCondition) {
    this.editedCondition = condition;
  }

  finishEditingCondition() {
    if(this.editedCondition == undefined) {
      return;
    }

    if(this.editedCondition!.validate()) {
      if(this.editedCondition === this.addedCondition) {
        this.gamesytemConditionChangeEmitter.emit(this.editedCondition);
        this.addedCondition = undefined
      }
      this.editedCondition = undefined;
    } else {
      this.snackbar.open("Invalid Condition", "", {duration: 2000});
    }
  }

  deleteCondition(condition: GamesystemCondition) {
    this.gamesystemConditions = this.gamesystemConditions.filter(c => c !== condition);
    this.datasource.data = this.gamesystemConditions;
    this.gamesystemConditionDeleteEmitter.emit(condition);
  }

  addCondition() {
    const condition = new GamesystemCondition(CharacterDependency.NONE, undefined, undefined);
    this.gamesystemConditions.push(condition);
    this.editedCondition = condition;
    this.addedCondition = condition;

    this.datasource.data = this.gamesystemConditions;
  }

  protected readonly CharacterDependency = CharacterDependency;
}
